@@ -1154,13 +1154,8 @@ static int esmtp_auth(struct esmtp_info *info,
 
 		if (!auth_key || !q)	continue;
 
-#if	HAVE_STRCASECMP
 		if (strcasecmp(q, auth_key) == 0)
 			break;
-#else
-		if (stricmp(q, auth_key) == 0)
-			break;
-#endif
 	}
 	fclose(configfile);
 
@@ -1913,7 +1908,7 @@ static char *esmtp_mailfrom_cmd_idna(struct esmtp_info *info,
 	if (info->hasverp && mf_info->verp)
 		verpverb=" VERP";	/* ESMTP VERP capability */
 
-	if (info->hassmtputf8)
+	if (info->hassmtputf8 && mf_info->issmtputf8)
 		smtputf8verb=" SMTPUTF8";
 	else
 	{
