@@ -27,6 +27,8 @@
 #include	"sysconfdir.h"
 #include	<courierauth.h>
 #include	<idna.h>
+#include	"numlib/numlib.h"
+#include	"comuidgid.h"
 
 #if	HAVE_SYSLOG_H
 #include	<syslog.h>
@@ -206,7 +208,7 @@ static void rw_del_local2(struct rw_info *rwi,
 	int	i;
 	struct localauthinfo lai;
 #if LOCAL_EXTENSIONS
-	char *atdomain;
+	char *atdomain, *localat;
 #endif
 
 	if (strchr(addr, '!') || *addr == 0)
@@ -238,7 +240,7 @@ static void rw_del_local2(struct rw_info *rwi,
 	}
 
 #if LOCAL_EXTENSIONS
-	atdomain=strrchr(addr, '@');
+	localat=atdomain=strrchr(addr, '@');
 	if (atdomain)
 		*atdomain++=0;
 #endif
@@ -355,8 +357,8 @@ static void rw_del_local2(struct rw_info *rwi,
 	}
 
 #if LOCAL_EXTENSIONS
-	if (atdomain)
-		atdomain[-1]='@';	/* Put back what was taken */
+	if (localat)
+		*localat='@';	/* Put back what was taken */
 #endif
 
 	/* Try to deliver to an alias defined in ${sysconfdir}/aliasdir */
@@ -364,8 +366,8 @@ static void rw_del_local2(struct rw_info *rwi,
 toalias:
 
 	{
-	struct	authinfo aa;
-	static const uid_t	mailuid=MAILUID;
+		struct	authinfo aa;
+		uid_t	mailuid=MAILUID;
 
 		memset(&aa, 0, sizeof(aa));
 		aa.homedir= ALIASDIR;
