@@ -28,15 +28,9 @@
 #if	HAVE_SYS_STAT_H
 #include	<sys/stat.h>
 #endif
-#if TIME_WITH_SYS_TIME
-#include <sys/time.h>
-#include <time.h>
-#else
+#include	<time.h>
 #if HAVE_SYS_TIME_H
-#include <sys/time.h>
-#else
-#include <time.h>
-#endif
+#include	<sys/time.h>
 #endif
 #include	<stdio.h>
 #include	<stdlib.h>
@@ -60,7 +54,17 @@
 #define POP3DLIST "courierpop3dsizelist"
 #define LISTVERSION 3
 
+#ifndef RUNTIME_START
+#define RUNTIME_START time(NULL)
+#endif
+
+#ifndef RUNTIME_CUR
+#define RUNTIME_CUR time(NULL)
+#endif
+
 extern void pop3dcapa();
+extern void pop3dlang(const char *);
+
 static void acctout(const char *disc);
 void rfc2045_error(const char *p)
 {
@@ -188,7 +192,7 @@ static struct msglist **readpop3dlist(unsigned long *uid)
 	int vernum=0;
 	unsigned long size;
 
-	uidv=time(NULL);
+	uidv=RUNTIME_CUR;
 
 	if (fp == NULL ||
 	    fgets(linebuf, sizeof(linebuf)-1, fp) == NULL ||
@@ -297,6 +301,9 @@ static int savepop3dlist(struct msglist **a, size_t cnt,
 
 	struct maildir_tmpcreate_info createInfo;
 
+#ifdef SAVEHOOK
+	SAVEHOOK();
+#endif
 	maildir_tmpcreate_init(&createInfo);
 
 	createInfo.uniq="pop3";
@@ -486,7 +493,9 @@ static void sortmsgs()
 	}
 
 	if (prev_list[n])
+	{
 		savesizes=1;
+	}
 
 	for (i=0; prev_list[i]; i++)
 	{
@@ -726,7 +735,7 @@ static void do_retr(unsigned i, unsigned *lptr)
 
 	if (msglist_a[i]->isutf8 && !utf8_enabled)
 	{
-		sprintf(boundary, "=_%d-%d", (int)getpid(), (int)time(NULL));
+		sprintf(boundary, "=_%d-%d", (int)getpid(), (int)RUNTIME_CUR);
 
 		sprintf(wrapheader, MIMEWRAPTXT, boundary, boundary, boundary,
 			mime_message_type);
@@ -908,7 +917,7 @@ static void acctout(const char *disc)
 
 	libmail_str_size_t(top_count, num1);
 	libmail_str_size_t(retr_count, num2);
-	libmail_str_time_t(time(NULL)-start_time, num3);
+	libmail_str_time_t(RUNTIME_CUR-start_time, num3);
 	libmail_str_size_t(bytes_received_count, numAR);
 	libmail_str_size_t(bytes_sent_count, numAS);
 
@@ -943,13 +952,10 @@ static void acctout(const char *disc)
 	free(p);
 }
 
-static RETSIGTYPE bye(int signum)
+static void bye(int signum)
 {
 	acctout("INFO: TIMEOUT");
 	exit(0);
-#if	RETSIGTYPE != void
-	return (0);
-#endif
 }
 
 static void loop()
@@ -1010,6 +1016,12 @@ int	c;
 			continue;
 		}
 
+		if (strcmp(p, "LANG") == 0)
+		{
+			pop3dlang(strtok(NULL, "\r\n"));
+			continue;
+		}
+
 		if (strcmp(p, "DELE") == 0)
 		{
 		unsigned	i;
@@ -1113,11 +1125,10 @@ char	*n;
 int main(int argc, char **argv)
 {
 char	*p;
-
 #ifdef HAVE_SETVBUF_IOLBF
 	setvbuf(stderr, NULL, _IOLBF, BUFSIZ);
 #endif
-	time(&start_time);
+	start_time=RUNTIME_START;
 
 	if ((authaddr=getenv("AUTHADDR")) == NULL ||
 	    *authaddr == 0)
