@@ -927,6 +927,7 @@ static int do_msgmove(const char *from,
 			{
 				free(destdir);
 				enomem();
+				exit(0); /* gcc warning fix */
 			}
 			strcat(strcpy(p, destdir), "/shared");
 			free(destdir);
@@ -1122,30 +1123,7 @@ static void maildir_checknew(const char *folder, const char *dir)
 
 	/* Move everything from new to cur */
 
-	dirbuf=alloc_filename(dir, "new", "");
-
-	for (dirp=opendir(dirbuf); dirp && (dire=readdir(dirp)) != 0; )
-	{
-	char	*oldname, *newname;
-	char	*p;
-
-		if (dire->d_name[0] == '.')	continue;
-
-		oldname=alloc_filename(dirbuf, dire->d_name, "");
-
-		newname=malloc(strlen(oldname)+4);
-		if (!newname)	enomem();
-
-		strcat(strcat(strcpy(newname, dir), "/cur/"), dire->d_name);
-		p=strrchr(newname, '/');
-		if ((p=strchr(p, ':')) != NULL)	*p=0;	/* Someone screwed up */
-		strcat(newname, ":2,");
-		rename(oldname, newname);
-		free(oldname);
-		free(newname);
-	}
-	if (dirp)	closedir(dirp);
-	free(dirbuf);
+	maildir_getnew(dir, 0, NULL, NULL);
 
 	/* Look for any messages mark as deleted.  When we delete a message
 	** we link it into the Trash folder, and mark the original with a T,
@@ -3069,23 +3047,24 @@ static void maildir_save_start(const char *folder,
 	save_time=t;
 #if 1
 	{
-	  int f = -1;
-	  char *tmpfname = alloc_filename(maildir,
-					   "", MAILDIRCURCACHE ".nfshack");
-	  if (tmpfname) {
-	    f = open(tmpfname, O_CREAT|O_WRONLY, 0600);
-	    free(tmpfname);
-	  }
-	  if (f != -1) {
-	    struct stat s;
-	    if (write(f, ".", 1) != 1)
-		    ; /* ignore */
-	    fsync(f);
-	    if (fstat(f, &s) == 0)
-	      save_time = s.st_mtime;
-	    close(f);
-	    unlink(tmpfname);
-	  }
+		int f = -1;
+		char *tmpfname = alloc_filename(maildir,
+						"", MAILDIRCURCACHE ".nfshack");
+		if (tmpfname) {
+			f = open(tmpfname, O_CREAT|O_WRONLY, 0600);
+
+			if (f != -1) {
+				struct stat s;
+				if (write(f, ".", 1) != 1)
+					; /* ignore */
+				fsync(f);
+				if (fstat(f, &s) == 0)
+					save_time = s.st_mtime;
+				close(f);
+				unlink(tmpfname);
+			}
+			free(tmpfname);
+		}
 	}
 #endif
 
