@@ -1066,12 +1066,12 @@ static char *convertcid(const char *cidurl, void *voidp)
 	else
 		p=(*cid_info->info->get_url_to_mime_part)(mimeid,
 							  cid_info->info);
-	free(mimeid);
-
 	if (*mimegpgfilename && rfc->parent &&
 	    libmail_gpgmime_is_decoded(rfc->parent, &dummy))
 		add_decoded_link(rfc->parent, mimeid, dummy);
 
+	free(mimeid);
+
 	return p;
 }
 
@@ -1322,7 +1322,7 @@ struct msg2html_textplain_info {
 	*/
 	int flowed;
 
-	int conv_err; /* A transcoding error has occured */
+	int conv_err; /* A transcoding error has occurred */
 
 	/*
 	** Optionally replace smiley sequences with image URLs
